/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.osgi.web.servlet.jsp.compiler.internal;

import com.liferay.portal.kernel.util.GetterUtil;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.atomic.AtomicInteger;

import javax.servlet.ServletConfig;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.JspTag;
import javax.servlet.jsp.tagext.Tag;

import org.apache.jasper.Constants;
import org.apache.jasper.runtime.TagHandlerPool;

/**
 * @author Shuyang Zhou
 * @author Preston Crary
 * @see    com.liferay.support.tomcat.jasper.runtime.TagHandlerPool
 */
public class JspTagHandlerPool extends TagHandlerPool {

	@Override
	public <T extends JspTag> JspTag get(Class<T> jspTagClass)
		throws JspException {

		JspTag jspTag = _jspTags.poll();

		if (jspTag == null) {
			try {
				jspTag = jspTagClass.newInstance();
			}
			catch (Exception e) {
				throw new JspException(e);
			}
		}
		else {
			_counter.getAndDecrement();
		}

		return jspTag;
	}

	@Override
	public void release() {
		JspTag jspTag = null;

		while ((jspTag = _jspTags.poll()) != null) {
			if (jspTag instanceof Tag) {
				Tag tag = (Tag)jspTag;

				tag.release();
			}
		}
	}

	@Override
	public void reuse(JspTag jspTag) {
		if (_counter.get() < _maxSize) {
			_counter.getAndIncrement();

			_jspTags.offer(jspTag);
		}
		else if (jspTag instanceof Tag) {
			Tag tag = (Tag)jspTag;

			tag.release();
		}
	}

	@Override
	protected void init(ServletConfig config) {
		_maxSize = GetterUtil.getInteger(
			getOption(config, OPTION_MAXSIZE, null), Constants.MAX_POOL_SIZE);
	}

	private final AtomicInteger _counter = new AtomicInteger();
	private final Queue<JspTag> _jspTags = new ConcurrentLinkedQueue<>();
	private int _maxSize;

}