/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.osgi.web.servlet.jsp.compiler.internal;

import com.liferay.portal.kernel.zip.ZipFileUtil;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

/**
 * @author Shuyang Zhou
 */
public class VfsJavaFileObject extends BaseJavaFileObject {

	public VfsJavaFileObject(String className, URL url, String entryName)
		throws MalformedURLException {

		super(Kind.CLASS, className);

		_entryName = entryName;

		String file = url.getFile();

		int index = file.indexOf(".jar");

		if (index < 0) {
			throw new MalformedURLException(
				url + " does not denote a jar file");
		}

		_file = new File(file.substring(0, index + 4));
	}

	@Override
	public InputStream openInputStream() throws IOException {
		return ZipFileUtil.openInputStream(_file, _entryName);
	}

	@Override
	public URI toUri() {
		return _file.toURI();
	}

	private final String _entryName;
	private final File _file;

}